﻿using System.Collections;
using System.ComponentModel;

namespace TestProject
{
    /// <summary>
    /// 자동 필터 컬럼
    /// </summary>
    /// <typeparam name="TItem">항목 타입</typeparam>
    public class AutoFilterColumn<TItem> : INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 이벤트 - PropertyChanged

        /// <summary>
        /// 속성 변경시 이벤트
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 값
        /// </summary>
        private object value;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 부모 - Parent

        /// <summary>
        /// 부모
        /// </summary>
        public AutoFilterCollection<TItem> Parent { get; set; }

        #endregion
        #region 명칭 - Name

        /// <summary>
        /// 명칭
        /// </summary>
        public string Name { get; set; }

        #endregion
        #region 레벨 - Level

        /// <summary>
        /// 레벨
        /// </summary>
        public int? Level { get; set; }

        #endregion
        #region 값 - Value

        /// <summary>
        /// 값
        /// </summary>
        public object Value
        {
            get
            {
                return this.value;
            }
            set
            {
                if(this.value == value) 
                {
                    return;
                }

                this.value = value;

                Parent.NotifyAll();
            }
        }

        #endregion
        #region 중복 제거 값 열거 가능형 - DistinctValueEnumerable

        /// <summary>
        /// 중복 제거 값 열거 가능형
        /// </summary>
        public IEnumerable DistinctValueEnumerable
        {
            get
            {
                IEnumerable enumerable = Parent.GetValueEnumerableForFilter(Name);

                return enumerable;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Internal

        #region 통지하기 - Notify()

        /// <summary>
        /// 통지하기
        /// </summary>
        internal void Notify()
        {
            FirePropertyChangedEvent("DistinctValueEnumerable");
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 속성 변경시 이벤트 발생시키기 - FirePropertyChangedEvent(propertyName)

        /// <summary>
        /// 속성 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        protected void FirePropertyChangedEvent(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}