﻿using System;
using System.Reflection;

namespace TestProject.Extension
{
    /// <summary>
    /// 열거형 확장
    /// </summary>
    public static class EnumerationExtension
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 어트리뷰트 값 구하기 - GetAttributeValue<TAttribute, TAttributeValue>(value)

        /// <summary>
        /// 어트리뷰트 값 구하기
        /// </summary>
        /// <typeparam name="TAttribute">어트리뷰트 타입</typeparam>
        /// <typeparam name="TAttributeValue">어트리뷰트 값 타입</typeparam>
        /// <param name="value">열거형 값</param>
        /// <returns>어트리뷰트 값</returns>
        public static TAttributeValue GetAttributeValue<TAttribute, TAttributeValue>(this Enum value)
        {
            TAttributeValue attributeValue = default(TAttributeValue);

            if(value != null)
            {
                FieldInfo fieldInfo = value.GetType().GetField(value.ToString());

                if(fieldInfo != null)
                {
                    TAttribute[] attributeArray = fieldInfo.GetCustomAttributes(typeof(TAttribute), false) as TAttribute[];

                    if(attributeArray != null && attributeArray.Length > 0)
                    {
                        IAttribute<TAttributeValue> attribute = attributeArray[0] as IAttribute<TAttributeValue>;

                        if(attribute != null)
                        {
                            attributeValue = attribute.Value;
                        }
                    }
                }
            }

            return attributeValue;
        }

        #endregion
    }
}